rm(list=ls())

library(magrittr)
library(tidyverse)

response <- read_csv("laptops-hande.csv") %>% 
  mutate_if(is.character, factor)

contr.treatment(levels(response$Brand))
contr.sum(levels(response$Brand))

options("contrasts")
options(contrasts=c("contr.sum", "contr.poly"))
options("contrasts")
response

lm.res <- lm(-RANK ~ ., data= select(response, -Alternative))

partWorth0 <- coef(lm.res) %>% data.frame(partworth=.) %>% 
  rownames_to_column("feature") %>% 
  filter(feature!="(Intercept)") %>% 
  mutate(level=parse_number(feature),
         feature=str_extract(feature, "\\D*")) %>% ## regular expression
  select(feature, level, everything())
  
  
partWorth <- partWorth0 %>% 
  group_by(feature) %>% 
  summarize(level=as.integer(max(level)+1), partworth=-sum(partworth)) %>% 
  bind_rows(partWorth0) %>% 
  mutate(level=as.integer(level)) %>% 
  arrange(feature, level)

featureLevels <- response %>% 
  select_if(is.factor) %>% 
  map(levels) %>% 
  unlist() %>% 
  data.frame(level=.) %>% 
  rownames_to_column("feature")


partWorth <- partWorth %>% 
  mutate(temp=paste0(feature,level)) %>% 
  select(-level) %>% 
  left_join(featureLevels, by=c("temp"="feature")) %>% 
  select(-temp) %>% 
  select(feature, level, everything())

partImportance <- partWorth %>% 
  group_by(feature) %>% 
  summarise(range=diff(range(partworth))) %>% 
  mutate(importance=range/sum(range)) %>% 
  select(-range) %>% 
  arrange(desc(importance))

partImportance

partWorth %>% 
  mutate(feature=factor(feature,
                        levels=partImportance$feature)) %>% 
  ggplot(aes(level, partworth)) +
  geom_linerange(aes(ymin=0, ymax=partworth,
                     col=feature)) +
  geom_point(aes(col=feature)) +
  coord_flip() +
  facet_grid(feature~., scales="free", space="free")+
  theme(legend.position = "none", strip.text.y =element_text(angle=0)) +
  labs(y="Partworth", x=NULL)




